import {Component, Input, OnInit} from '@angular/core';
import {
  MatTableDataSource
} from "@angular/material/table";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {SimpleGamesystem} from "../../../../project/game-model/gamesystems/SimpleGamesystem";
import {ScriptAccount} from "../../../../project/game-model/scriptAccounts/ScriptAccount";
import {SimpleTransition} from "../../../../project/game-model/gamesystems/transitions/SimpleTransition";
import {SimpleState} from "../../../../project/game-model/gamesystems/states/SimpleState";
import {ScriptAccountCondition} from "../../../../project/game-model/gamesystems/conditions/ScriptAccountCondition";
import {TemplateElement} from "../../../../project/game-model/templates/TemplateElement";
import {
  SimpleTemplateTransition
} from "../../../../project/game-model/templates/simpleGamesystem/SimpleTemplateTransition";
@Component({
  selector: 'app-simple-transition-editor',
  templateUrl: './simple-transition-editor.component.html',
  styleUrl: './simple-transition-editor.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class SimpleTransitionEditorComponent implements OnInit {

  @Input() gamesystem: SimpleGamesystem | undefined
  @Input() scriptAccounts: ScriptAccount[] = []
  @Input() templateElement: TemplateElement | undefined

  displayedColumns: string[] = ["starting-state", "ending-state", "edit", "delete"];
  dataSource = new MatTableDataSource<SimpleTransition>();
  columnsToDisplayWithExpand = [...this.displayedColumns, 'expand'];
  expandedElement: SimpleTransition | null = null;
  editedTransition: SimpleTransition | undefined
  editedTransitionIndex = -1;

  defaultStartingState: SimpleState = new SimpleState("None", "");
  defaultEndingState: SimpleState = new SimpleState("None", "");

  transitionError: boolean = false;
  transitionStartingStateError = true;
  transitionEndingStateError = true;
  ngOnInit() {
    this.dataSource.data = this.gamesystem!.transitions;
    this.dataSource.filterPredicate = (data: SimpleTransition, filter: string) => {
      return [data.startingState, data.endingState].some((state) => state.stateLabel.toLowerCase().includes(filter))
    }
    console.log("TransitionEditor: ", this.scriptAccounts.length)
  }

  addTransition() {
    this.editedTransition = new SimpleTransition(this.defaultStartingState, this.defaultEndingState);
    const updatedData = this.dataSource.data;
    updatedData.push(this.editedTransition);
    this.dataSource.data = updatedData;
    this.editedTransitionIndex = this.dataSource.data.length;
  }

  validateEditedTransition() {
    this.transitionError = false;
    this.transitionStartingStateError = false;
    this.transitionEndingStateError = false;
    if(this.editedTransition!.startingState === this.editedTransition!.endingState) {
      this.transitionError = true;
    } else if(this.editedTransition!.startingState == this.defaultStartingState) {
      this.transitionStartingStateError = true;
    } else if(this.editedTransition!.endingState == this.defaultEndingState) {
      this.transitionEndingStateError = true;
    }
  }

  finishEditing() {
    this.validateEditedTransition();
    if(this.transitionError || this.transitionStartingStateError || this.transitionEndingStateError) {
      return;
    }
    this.gamesystem?.createTransition(this.editedTransition!.startingState, this.editedTransition!.endingState);
    this.dataSource.data = this.gamesystem!.transitions;
    this.editedTransition = undefined;
  }

  deleteTransition(transition: SimpleTransition) {
    if(this.gamesystem!.parentGamesystem == undefined) {
      this.gamesystem!.removeTransition(transition);
      this.dataSource.data = this.gamesystem!.transitions;
    }
  }

  applyFilter(event: KeyboardEvent) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
  }

  protected readonly transition = transition;

  onCreateCondition(transition: SimpleTransition, condition: ScriptAccountCondition) {
    if(this.templateElement != undefined && transition instanceof SimpleTemplateTransition) {
      transition.conditionMap.get(this.templateElement)!.push(condition)
    } else {
      transition.addScriptAccountCondition(condition);
    }

  }

  deleteCondition(trasition: SimpleTransition, condition: ScriptAccountCondition) {
    if(this.templateElement != undefined && trasition instanceof SimpleTemplateTransition) {
      let updatedConditions = trasition.conditionMap.get(this.templateElement)!
      updatedConditions = updatedConditions.filter(currentCondition => condition.scriptAccount !== currentCondition.scriptAccount);
      trasition.conditionMap.set(this.templateElement, updatedConditions);
    } else {
      console.log(this.templateElement)
      trasition.removeScriptAccountCondition(condition.scriptAccount);
    }

  }

  getDisplayedConditions(transition: SimpleTransition) {
    if(this.templateElement == undefined) {
      return transition.scriptAccountConditions
    } else if(transition instanceof SimpleTemplateTransition) {
      return transition.conditionMap.get(this.templateElement)!
    } else {
      return [];
    }
  }
}
